import pandas as pd
from tdxtrader.logger import logger
from tdxtrader.anis import RED, GREEN, YELLOW, BLUE, RESET

# 定义列名
COLUMNS = ['code', 'name', 'date', 'time', 'price', 'rate', 'value', 'sign']

# 需要修复的股票名称列表
STOCK_LIST = ['新 和 成', '五 粮 液', '农 产 品', '南 玻A', '万 科A', '新 华 都', '奥 特 迅', '三 力 士', '诺 普 信', '达 意 隆', '海 利 得', '全 聚 德', '怡 亚 通', '粤 传 媒', '红 宝 丽', '远 望 谷', '报 喜 鸟', '安 纳 达', '生 意 宝', '金 螳 螂', '兔 宝 宝', '南 京 港', '苏 泊 尔', '七 匹 狼', '新 大 陆', '中 关 村', '新 希 望', '张 裕A', '罗 牛 山', '鲁 泰A', '英 力 特', '柳 工', '渝 开 发', '金 融 街', '盐 田 港', '深 赛 格', '特 力A']

def fix_stock_name(line):
    """
    修复股票名称中的空格。
    """
    for stock in STOCK_LIST:
        fixed_stock = stock.replace(' ', '')  # 去掉空格
        line = line.replace(stock, fixed_stock)  # 替换原始行中的股票名称
    return line

def process_line(line):
    """
    处理每一行数据，确保列数为 8，并将 'price' 转换为 float 并保留两位小数。
    """
    line = line.strip()  # 去掉首尾空白字符
    line = fix_stock_name(line)  # 修复股票名称中的空格
    fields = line.split()  # 按空白字符分割

    # 检查列数是否为 8
    if len(fields) != 8:
        logger.error(f"【错误行】 {line}")
        return None  # 跳过该行

    # 将 'price' 转换为 float 并保留两位小数
    try:
        fields[4] = round(float(fields[4]), 2)  # price
    except ValueError as e:
        logger.error(f"【类型转换错误】 {line}: {e}")
        return None  # 跳过该行

    return fields

def read_file(file_path):
    """
    读取文件并处理错误。
    """
    try:
        # 检查文件是否为空
        with open(file_path, 'r', encoding='gbk') as file:
            first_line = file.readline()
            if not first_line:  # 文件为空
                return pd.DataFrame(columns=COLUMNS)

        # 逐行读取文件并处理
        rows = []
        with open(file_path, 'r', encoding='gbk') as file:
            for line in file:
                processed_line = process_line(line)  # 处理每一行
                if processed_line:  # 如果处理成功
                    rows.append(processed_line)

        # 构建 DataFrame
        df = pd.DataFrame(rows, columns=COLUMNS)
        return df

    except Exception as e:
        logger.error(f"读取文件时发生错误: {e}")
        return None
    
def clear_file_content(file_path):
    try:
        with open(file_path, 'w', encoding='gbk') as file:
            file.truncate(0)
            # 写入表头
            # header = ' '.join(COLUMNS) + '\n'
            # file.write(header)
        logger.debug(f"【重置文件】内容已清空")
    except Exception as e:
        logger.error(f"清空文件内容时发生错误: {e}")


from xtquant import xtconstant
import time
import math
import inspect
from tdxtrader.file import read_file
from tdxtrader.utils import add_stock_suffix, timestamp_to_datetime_string, convert_to_current_date
from tdxtrader.anis import RED, GREEN, YELLOW, BLUE, RESET
from tdxtrader.logger import logger
from tdxtrader.trader import error_orders

def get_volume(paload, row):
    if paload.get('size') is not None:
        return paload.get('size')
    elif paload.get('amount') is not None:
        if paload.get('price') > 0:
            return math.floor(paload.get('amount') / paload.get('price') / 100) * 100
        else:
            return math.floor(paload.get('amount') / row.get('price') / 100) * 100
    else:
        return 100
    

def get_position(positions, stock_code):
    for position in (positions or []):
        if position.stock_code == stock_code:
            return position
    return None

def create_order(xt_trader, account, file_path, previous_df, buy_sign, sell_sign, buy_event, sell_event):
    current_df = read_file(file_path)
    if current_df is not None:
        if previous_df is not None:
            # 比较前后两次读取的 DataFrame，找出新增的行
            new_rows = current_df[~current_df.index.isin(previous_df.index)]
            if not new_rows.empty:
                for index, row in new_rows.iterrows():

                    stock_code = add_stock_suffix(row['code'])

                    price_type_map = {
                        '市价': xtconstant.LATEST_PRICE,
                        '限价': xtconstant.FIX_PRICE
                    }

                    positions = xt_trader.query_stock_positions(account)

                    position = get_position(positions, stock_code)

                    params = {
                        'xt_trader': xt_trader,
                        'account': account,
                        'stock': row,
                        'position': position
                    }
                    
                    if row['sign'] == buy_sign:
                        if len(inspect.signature(buy_event).parameters) > 1: # 检查 buy_event 是否需要额外的参数
                            buy_paload = buy_event(row, position, xt_trader)
                        else:
                            buy_paload = buy_event(params)
                        if buy_paload is not None:
                            xt_trader.order_stock_async(
                                account=account, 
                                stock_code=stock_code, 
                                order_type=xtconstant.STOCK_BUY, 
                                order_volume=get_volume(buy_paload, row), 
                                price_type=price_type_map.get(buy_paload.get('type')) or xtconstant.LATEST_PRICE,
                                price=buy_paload.get('price') or -1,
                                order_remark=row.get('name')
                            )
                    elif row['sign'] == sell_sign:
                        if position is not None:
                            if len(inspect.signature(sell_event).parameters) > 1: # 检查 sell_event 是否需要额外的参数
                                sell_paload = sell_event(row, position, xt_trader)
                            else:
                                sell_paload = sell_event(params)
                                
                            if sell_paload is not None:
                                xt_trader.order_stock_async(
                                    account=account, 
                                    stock_code=stock_code, 
                                    order_type=xtconstant.STOCK_SELL, 
                                    order_volume=get_volume(sell_paload, row),
                                    price_type=price_type_map.get(sell_paload.get('type')) or xtconstant.LATEST_PRICE,
                                    price=sell_paload.get('price') or -1,
                                    order_remark=row.get('name')
                                )
                        else:
                            logger.warning(f"{YELLOW}【无持仓】{RESET}没有查询到持仓信息，不执行卖出操作。股票代码：{stock_code}, 名称：{row['name']}")
                
        return current_df
    
    return None

def cancel_order(xt_trader, account, cancel_after):
    if cancel_after is not None:
        orders = xt_trader.query_stock_orders(account, cancelable_only=True)
        for order in orders:
            if order.order_id in error_orders:
                return
            order_time = convert_to_current_date(order.order_time)
            if time.time() - order_time >= cancel_after:
                xt_trader.cancel_order_stock_async(account, order.order_id)